// $Header: p:\\graphics\\CVS/src/GraphicsTown/FlyCamera.cpp,v 1.11 2000/11/27 19:39:42 gleicher Exp $
// GraphicsTown - CS559-2000, sample solution
//
#include "GrTown_PCH.H"

#include "FlyCamera.H"
#include "DrawUtils.H"
#include <time.h>

FlyCamera::FlyCamera()
{
  ridable = true;
  transMatrix(transform,0,10,0);
  name = "FlyCamera";
  lastUItime = 0;
}
void FlyCamera::draw(DrawingState*)
{
}

int FlyCamera::handle(int e)
{
	static int stX, stY;
	static int lX, lY;

	switch(e) {
	case FL_PUSH:
		lX = stX = Fl::event_x();
		lY = stY = Fl::event_y();
		return 1;
	case FL_RELEASE:
		return 1;
	case FL_FOCUS:
		return 1;
	case FL_KEYBOARD:
		switch(Fl::event_key()) {
		case 'z':
			// cross Y with Z to get X
		  cross(0,1,0,
				transform[2][0],transform[2][1],transform[2][2],
				transform[0][0],transform[0][1],transform[0][2]);
		  cross(transform[2][0],transform[2][1],transform[2][2],
				transform[0][0],transform[0][1],transform[0][2],
				transform[1][0],transform[1][1],transform[1][2]);

			break;
		};
		return 1;
	case FL_DRAG:
		int mX = Fl::event_x();
		int mY = Fl::event_y();
	
		switch(Fl::event_button()) {
		case 1:
		  break;
		};

		lX = mX; lY = mY;
		return 1;
	};
	return 0;
}

void FlyCamera::turn(float a)
{
	Matrix tmp,newCam;
	rotMatrix(tmp,'Y',a);
	multMatrix(tmp,transform,newCam);
	copyMatrix(newCam,transform);
}
void FlyCamera::pitch(float a)
{
	Matrix tmp,newCam;
	rotMatrix(tmp,'X',a);
	multMatrix(tmp,transform,newCam);
	copyMatrix(newCam,transform);
}
void FlyCamera::forward(float a)
{
	  transform[3][0] -= a*transform[2][0];
	  transform[3][1] -= a*transform[2][1];
	  transform[3][2] -= a*transform[2][2];
}
void FlyCamera::localTrans(float x,float y,float z)
{
	  transform[3][0] -= x*transform[0][0] + y*transform[1][0] + z*transform[2][0];
	  transform[3][1] -= x*transform[0][1] + y*transform[1][1] + z*transform[2][1];
	  transform[3][2] -= x*transform[0][2] + y*transform[1][2] + z*transform[2][2];
}
bool FlyCamera::uiStep()
{
  unsigned long uitime = clock();
  unsigned long te = uitime-lastUItime;
  lastUItime = uitime;

  float u = ((float) te) / 30.f;

  bool didSomething = false;
  Matrix tmp,tmp2;
  Matrix newCam;
  idMatrix(tmp);
  idMatrix(tmp2);

  if (Fl::get_key(FL_Left))
	turn(.05f * u);
  if (Fl::get_key(FL_Right))
	turn(-.05f * u);
  if (Fl::get_key(FL_Up))
	pitch(-.05f * u);
  if (Fl::get_key(FL_Down))
	pitch(.05f * u);
  if (Fl::get_key(' '))
	forward(3 * u);
  if (Fl::get_key('x'))
	  forward(-2 * u);
  if (Fl::get_key(FL_KP+'8'))
	  localTrans(0,-u,0);
  if (Fl::get_key(FL_KP+'2'))
	  localTrans(0,u,0);
  if (Fl::get_key(FL_KP+'4'))
	  localTrans(u,0,0);
  if (Fl::get_key(FL_KP+'6'))
	  localTrans(-u,0,0);
  if (Fl::get_key(FL_KP+'1'))
	  turn(.05f * u);
  if (Fl::get_key(FL_KP+'3'))
	  turn(-.05f * u);
  if (Fl::get_key(FL_KP+'5'))
	  localTrans(0,0,u);
  if (Fl::get_key(FL_KP+'0'))
	  localTrans(0,0,-u);

  multMatrix(tmp2,transform,newCam);
  copyMatrix(newCam,transform);
  orthogonalize(transform);
  return didSomething;
}

FollowCam::FollowCam() : GrObject("follower"), following(0)
{
  ox = oz = 0;
  oy = 20;
  ridable = 1;
  lastUItime = 0;
  odx = odz = 0;
  ody = 20;
}

void FollowCam::getTransform()
{
  float x=0;
  float y=0;
  float z=0;

  if (following) {
	x = following->transform[3][0];
	y = following->transform[3][1];
	z = following->transform[3][2];
  }

  if (x==ox) {
	ox = ox-1;
  }
  if (z==oz) {
	oz = oz-1;
  }
  float mdst = (ABS(x-ox) + ABS(z-oz));
  if (mdst < 20) {
	if (ABS(x-ox) < 15) {
	  ox += ((x-ox) < 0) ? 1 : -1;
	}
	if (ABS(z-oz) < 15) {
	  ox += ((z-oz) < 0) ? 1 : -1;
	}
  } else if (mdst > 50) {
	if (ABS(x-ox) > 30)
	  ox += .1f*(x-ox);
	else
	  ox += ((x-ox) < 0) ? -1 : 1;
	if (ABS(z-oz) > 30)
	  oz += .1f*(z-oz);
	else
	  oz += ((z-oz) < 0) ? -1 : 1;
  }

  ox += (x + odx); ox /= 2;
  oy += (y + ody); oy /= 2;
  oz += (z + odz); oz /= 2;

	lookatMatrix(ox,oy,oz, x,y,z, transform);

  odx = ox - x;
  ody = oy - y;
  odz = oz - z;
}

bool FollowCam::uiStep()
{  unsigned long uitime = clock();
  unsigned long te = uitime-lastUItime;
  lastUItime = uitime;

  float u = ((float) te) / 20.f;

  if (Fl::get_key(FL_Left))
  { ox += u; odx += u; }
  if (Fl::get_key(FL_Right)) {
	  ox -= u; odx -=u; }
  if (Fl::get_key(FL_Up)) {
	  oy += u; ody += u; }
  if (Fl::get_key(FL_Down)) {
	oy -= u; ody -=u;
  }

  return true;
}

int FollowCam::handle(int e)
{
	switch(e) {
	case FL_FOCUS:
		return 1;
	case FL_KEYBOARD:
//		switch(Fl::event_key()) {
//		};
		return 1;
	};
	return 0;
}
