#ifndef GrObject_H
#define GrObject_H
// $Header: p:\\graphics\\CVS/src/GraphicsTown/GrObject.H,v 1.10 2000/11/27 10:30:06 gleicher Exp $
// GraphicsTown - CS559-2000, sample solution
// (c) 2000 Michael L. Gleicher
//
// This example code should give you some idea as to how my system
// worked.


#include "MMatrix.H"

// class GrObject: a drawable object in the word of GraphicsTown

// other classes referred to:
class DrawingState;
class Behavior;

extern int noNameCtr;
class GrObject {
 public:
  GrObject(char* nameBase = "NoName", int& nameCtr = noNameCtr, char* xn=0);
  virtual ~GrObject();

  // things to know about the object
  bool ridable;					/* can we "ride" this thing */
  char* name;
  float rideAbove;				// in case we don't want to be stuck on the ground

  // simulation: every object is required to "update itself" to the
  // current time. no guaruntees that this clock is updated
  // uniformly (since it depends on drawing rate). The units are
  // number of milliseconds since the beginning of the simulation
  virtual void simulateUntil(unsigned long time);

  // what makes the object go - by default, this is all that
  // SimulateUntil does.
  // note - this is a linked list! All behaviors get their chance
  Behavior* behaviors;

  // the main thing an object does is draw itself
  // this happens in 2 phases, one where normal drawing happens, and
  // one phase that is "after" all the other objects have draw. This
  // allows for transparent objects to be drawn last. 
  // The caller will always take care of doing a
  // pushAttributes/matrix, so don't worry about that
  virtual void draw(DrawingState*);
  virtual void drawAfter(DrawingState*);

  // most objects have some way to position themselves
  // by standardizing that, we can make any object "ridable"
  virtual void getTransform();			// fill in this matrix
  Matrix transform;

  // if we are "riding" this object (that means that it is the current
  // camera), it has two responsibilities:
  // 1) it must set up the camera transformation (not the projection)
  // 2) it must provide the user interface
  virtual bool setTransform();	// if you can't do it, say so
  virtual bool uiStep();		// called if you're the camera before
								// drawing - lets you do key polling
  virtual int handle(int e);	// exactly what the FlTk Widgets do

  // we will keep a linked list of these things - and I hate STL
  GrObject* next;

  // we also have hierarchy
  GrObject* parent;				// your parent is responsible for you
  GrObject* children;
  GrObject* add(GrObject*, float x=0, float y=0, float z=0, float ry=0);
};

// add a new object onto the objects list
// allow for positioning in case object doesn't
// note: if you want to make a subobj, use the method of GrObj
extern GrObject* theObjects;
GrObject* add(GrObject*, float x=0, float y=0, float z=0, float ry=0);

// draw a list of objects
void drawObList(GrObject* head, DrawingState* drst);
void drawAfterObList(GrObject* head, DrawingState* drst);

#endif
