#ifndef ROADS_H
#define ROADS_H

#include "GrObject.H"
#include "Behaviors.H"

class Intersection : public Road {
 public:
  int id;
  Intersection(int nLanes);
  virtual void roadDraw();
  int nLanes;

  // position of center
  bool cachedPosition;
  float pcacheX, pcacheZ;
  void getPosition(float& x, float& z);

  // what connects to this
  // 0 = [1,0], 1=[0,1], 2=[-1,0], 3=[0,-1]
  int neighbor[4];
};

// convert dx,dz to a direction number
// or vice versa
int direction(float,float);
void direction(int, float&, float&);

typedef vector<Intersection*> IntersectionVector ;

extern IntersectionVector theIntersectionsV;

inline Intersection* theIntersections(int i)
{
	if (i<1) {
		printf("illegal intersection %d\n",i);
		return 0;
	} else
	return theIntersectionsV[i-1];
}

// a behavior that follows the roads
typedef enum {driving, stopped, intersection,turning} CruiseState;
class Drive : public Behavior {
 public:
  Drive(GrObject* car, int startInt, int startDir=0);

  int lastIntersect;			/* where were we */
  int nextIntersect;			/* where are we going */
  int dir, newDir;
  
  unsigned long stateStart;
  CruiseState state;
  unsigned long delay;

  int genNext(int from, int oldDir, int& newDir);

  //
  virtual void simulateUntil(unsigned long);

  // interpolate between two places
  float fromX, fromZ;
  float toX, toZ;

  // find the location of the intersection entry/exit
  void locateEntry(int intersect, int dir, float& x, float& z);
  void locateExit(int intersect, int dir, float& x, float& z);

  // set the matrix of the object to something corresponding to...
  // notice that this takes a float (not just an int) so we can interpolate
  void setMatrix(float x, float z, float dir);

  // get the position from the current
  void getCurPos(float&, float&);

  // compute how long to drive from here to there
  unsigned long driveTime();
};

#endif
